import json
from collections import OrderedDict

from django.contrib.auth.models import User
from django.db import models
from djangotools.common.path import DeferredModelPath
from djangotools.models.serializedmodel import SerializableModel
from djangotools.models import UserIdManager

from djangotools.common.auto_path import AutoPath

class Pref:
    def __init__(self, name, default):
        self.name = name
        self.default = default

class UserPreferenceManagerMeta(type):
    def __new__(cls, *args, **kwargs):
        obj = super().__new__(cls, *args, **kwargs)
        if isinstance(obj.preferences, (list, tuple)):
            obj.preferences_dict = OrderedDict([(x.name, x) for x in obj.preferences])
        elif isinstance(obj.preferences, dict):
            obj.preferences_dict = OrderedDict([(k, Pref(k, v)) for k, v in obj.preferences.items()])
        return obj

class UserPreferenceManager(metaclass=UserPreferenceManagerMeta):
    preferences = []
    def __init__(self, user, profile="default"):
        self.user=user
        self.profile=profile

    def update(self, data):
        for k, v in data.items():
            self.set(k, v)

    def set(self, key, value):
        pref = self._get(key)
        if pref:
            pref.set(value)
            return value
        Preference.objects.create(
                user=self.user,
                profile=self.profile,
                key=key,
                value = json.dumps(value)
            )
        return value

    def get(self, key, default=None):
        pref = self._get(key)
        if pref is not None:
            return pref.get()
        if key in self.preferences_dict:
            return self.set(key, self.preferences_dict[key].default)
        return self.set(key, default)


    def _get(self, key):
        try:
            return Preference.objects.get(user=self.user, profile=self.profile, key=key)
        except Preference.DoesNotExist:
            return None

    @property
    def dict(self):
        if not self.user.is_authenticated:
            return {}
        return {
            x.key : x.get() for x in Preference.objects.filter(user=self.user, profile=self.profile)
        }





class Preference(models.Model, SerializableModel, AutoPath):
    _path_ = None


    objects = UserIdManager()
    id : int
    user : User = models.ForeignKey(User, on_delete=models.CASCADE)
    profile : str = models.TextField(default="default")
    key : str = models.TextField()
    value : str = models.TextField(null=True, blank=True)

    class Meta:
        unique_together = ['user', 'profile', "key"]

    def set(self, value):
        self.value=json.dumps(value)
        self.save()

    def __str__(self):
        return f"{self.key} : {json.dumps(self.value)}  "

    def __repr__(self): return self.__str__()

    def get(self):
        return json.loads(self.value) if self.value is not None else None

Preference.register()